IF NOT VERSION()>=4
  THEN
    QUIT "CopyFld4 cannot be used with versions of Paradox earlier than 4.0"
ENDIF

PROC CopyFld4()
;Ŀ
;CopyFld4 1.1 (C)1990,1991,1992,1993 Data Works of Houston.All rights reserved
;   Written by: Cary Jensen of Data Works of Houston                          
;               (713) 359-3311                                                
;   Data Works of Houston specializes in Paradox consulting, training, and    
;   application development.                                                  
;   We can provide all your Paradox and Paradox for Windows solutions.        
;                                                                             
;                                                                             
;   This program permits you to copy and paste fields within and between      
;   Paradox Reports.                                                          
;   Here are two ways to play CopyFld4:                                       
;      1.  Press [ALT][F10] to display the PAL menu.  Select {Play} from the  
;          the PAL menu and then type                                         
;          READLIB "C:\\PDOX40\\CopyTool" CopyFld4 CopyFld4()                 
;          If you use this method, these steps must be repeated each time you 
;          want to invoke CopyFld4.  The following way is much easier.        
;      2.  Use the SETKEY command to allow you to play CopyFld4 by pressing a 
;          key sequence.  For example, to play CopyFld4 simply by pressing    
;          [CTRL][F3], you need to use the following steps only once during   
;          your Paradox session.                                              
;          Press [ALT][F10] to display the PAL menu.  Select MiniScript and   
;          enter the following command:                                       
;          SETKEY "F23" READLIB "C:\\PDOX40\\CopyTool" CopyFld4 CopyFld4()    
;          Note: in the above command, dual backslashes (\\) are necessary.   
;          If you use an init script, add this command to it and CopyFld4 will
;          automatically be available every time you start Paradox.           
;   Both of these examples assume that you have placed CopyTool.lib onto the  
;   path C:\PDOX40.  If you store CopyTool in any other directory, supply     
;   that directory name instead of C:\\PDOX40 when using either of the        
;   above methods to play CopyFld4.                                           
;                                                                             
; The following array elements are not released.  They are used by the system 
; to remember what the copied field's contents are.                           
;     CF["Decimals"]       = # of decimal places                              
;     CF["WrappedTimes"]   = # of lines wrapped onto                          
;     CF["Justify"]        = type of justification                            
;     CF["Grouped"]        = is a summary field grouped                       
;     CF["PlaceType"]      = type of field                                    
;     CF["SummaryType"]    = is a summary field regular or calculated         
;     CF["NegParens"]      = are negative values enclosed in parenthese       
;     CF["StatType"]       = type of summary or report field                  
;     CF["Commas"]         = are commas used                                  
;     CF["NameOfField"]    = name of the field, calculation                   
;     CF["Currency"]       = is the field a currency field                    
;     CF["FieldFormat"]    = field string as it appears on the report         
;     CF["Alpha"]          = is the field an alphanumeric field               
;     CF["AlwaysSign"]     = is the field always signed                       
;     CF["Digits"]         = number of significant digits                     
;     CF["International"]  = is international notation used                   
;     CF["PointNoPennies"] = field uses a decimal point but no decimal places 
;     CF["Wrapped"]        = is the alpha field word wrapped                  
;                                                                             
;   Use the Reset selection from the CopyFld4 menu to release this array.     
;                                                                             
;   This utility is offered to Paradox users free of charge.                  
;   You may give copies of CopyFld4 to any other Paradox users who have a     
;   use for it as long as these copies have not been modified in any way.     
;                                                                             
;                      < < < <   DISCLAIMER   > > > > >                       
;   Data Works of Houston does not guarantee the suitability of CopyFld4 for  
;   any purpose and cannot be held liable for any damage caused either        
;   directly or indirectly through the use of CopyFld4.  By using CopyFld4 you
;   agree to these terms                                                      
;
PRIVATE Errorproc, Choice, StartingColumn, i, CurrentColumn, Position,
        MenuMove, StartColumn, TempDigits, Insert, NameLength, ChoiceKey,
        a, VerInfo, IsStandardMode
ErrorProc = "CopyFldError"

IF MEMLEFT() < 30000
   THEN
      QUIT "Not enough memory available to use CopyFld4."
ENDIF

STYLE ATTRIBUTE 79
@ 24,61 ?? " Loading CopyFld4"
STYLE ATTRIBUTE 207
?? "..."
STYLE

PROC CopyFldError()
   MENU ESC
   QUIT "Sorry, CopyFld4 cannot work in this situation."
ENDPROC ; --- CopyFldError()

PROC GetFieldSize()
  ;Determine the number of characters in the field mask
  Private DecimalPoint, WholeNumberSeparator, TakeOff, LeftOn, Temp
  CF["Digits"] = 0
  CF["Decimals"] = 0

  SWITCH
     CASE MATCH(UPPER(CF["FieldFormat"]),"..m.."):
        ;This is a date or time field, do nothing.
        Return
     CASE MATCH(CF["FieldFormat"],"..a.."):
        ;This is an alphanumeric field.  Set CF["Digits"] to length
        CF["Digits"] = LEN(CF["FieldFormat"])
        RETURN
     OTHERWISE:
        MENU {Field} {Reformat} ENTER
        IF MenuChoice() = ""
           THEN
              ;Report fields Page or #Record is selected, set CF["Digits"] to length.
              MENU ESC
              CF["Digits"] = LEN(CF["FieldFormat"])
              RETURN
           ELSE
              ;See if the numeric or currency field is in InternationalConvention
             {International}
             CF["International"] = MenuChoice() = "InternationalConvention"
             MENU ESC
        ENDIF
  ENDSWITCH
  ;Continue here only with numeric or Currency fields
  IF CF["International"]
     THEN
        DecimalPoint = ","
        WholeNumberSeparator = "."
     ELSE
        DecimalPoint = "."
        WholeNumberSeparator = ","
  ENDIF

  CF["Commas"] = NOT (SEARCH(WholeNumberSeparator, CF["FieldFormat"]) = 0)
  LeftOn = CF["FieldFormat"]
  TakeOff = " "

  WHILE LEN(LeftOn) > 0
     Temp = MATCH(LeftOn,"@..", Takeoff, Lefton)
     SWITCH
        CASE TakeOff = "9":
           CF["Digits"] = CF["Digits"] + 1
        CASE TakeOff = DecimalPoint:
           CF["PointNoPennies"] = NOT MATCH(LeftOn,"..9..")
           QUITLOOP
        OTHERWISE:
     ENDSWITCH
  ENDWHILE

  WHILE LEN(LeftOn)>0
     Temp = MATCH(LeftOn,"@..",Takeoff, LeftOn)
     SWITCH
        CASE TakeOff = "9":
           CF["Decimals"] = CF["Decimals"] + 1
        OTHERWISE:
     ENDSWITCH
  ENDWHILE
ENDPROC ;--- GetFieldSize()

Proc GroupedStatus()
  ;Return True if summary field is by group or overall
  IF LEN(CF["NameOfField"]) > 10
     THEN
        IF SUBSTR(CF["NameOfField"],LEN(CF["NameOfField"])-10,
           Len(CF["NameOfField"])) =", per group"
           THEN
              CF["NameOfField"]=SUBSTR(CF["NameOfField"], 1, LEN(CF["NameOfField"])-11)
              RETURN True
           ELSE
              RETURN False
        ENDIF
     ELSE
        RETURN False
  ENDIF
ENDPROC ;---GroupedStatus()

PROC SummaryType()    ;check for a [ or a "
 IF SEARCH("[", CF["NameOfField"])=0  AND SEARCH("\"", CF["NameOfField"])=0
    THEN
       RETURN "Regular"
    ELSE
       IF MATCH(CF["NameOfField"],"[..->..]")
         THEN
           RETURN "Regular"
         ELSE
           RETURN "Calculated"
       ENDIF
 ENDIF
ENDPROC ;--- SummaryType()

PROC WaitKey()
  Private Temp
  ;Wait for a keypress
  WHILE CHARWAITING()
    Temp = GETCHAR()
  ENDWHILE
  Temp = GETCHAR()
ENDPROC ;--- WaitKey()

Proc IsRegular()
  Private CurrentItem, LastItem, DashPosition, FldName
  ;First test if this field is a linked field from another directory
  IF MATCH(CF["NameOfField"],"[..->..]")
    THEN
      ;Get the position of the -> in the field name
      DashPosition = SEARCH("->",CF["NameOfField"])
      ;Select the table part first     
      SELECT SUBSTR(CF["NameOfField"],1,DashPosition+1)+"]"
      ;Get the field name      
      FldName = SUBSTR(CF["NameOfField"],DashPosition+2,
                LEN(CF["NameOfField"])-DashPosition-2)
      ;Now position the cursor on the field name
      CurrentItem = MenuChoice()
      WHILE UPPER(FldName) <> UPPER(MenuChoice())
         LastItem = MenuChoice()
         KEYPRESS MenuMove
         IF MenuChoice() = CurrentItem OR ;Ring menu returned to first choice
           MenuChoice() = LastItem        ;or standard menu remained on last
                                          ;item indicates regular field not found
           THEN
             CF["PlaceType"] = "Calculated"
             RETURN False
         ENDIF
      ENDWHILE
      ;The linked field item is selected
      ENTER
      RETURN True
  ENDIF
  ;The field is not a linked fields from another directory.
  ;This procedure verifies that the field is a regular field,
  ;not a calculation containing a variable or constant.
  CurrentItem = MenuChoice()
  WHILE UPPER(CF["NameOfField"]) <> UPPER(MenuChoice())
    LastItem = MenuChoice()
    KEYPRESS MenuMove
    IF MenuChoice() = CurrentItem OR ;Ring menu returned to first choice
       MenuChoice() = LastItem       ;or standard menu remained on last
                                    ;item indicates regular field not found
      THEN
        CF["PlaceType"] = "Calculated"
        RETURN False
    ENDIF
  ENDWHILE
  ENTER
  RETURN True
ENDPROC  ;--- IsRegular()

PROC ApplyJustification(AdjustCursorPosition)
  IF NOT CF["Justify"] = "None"
     THEN
        IF AdjustCursorPosition
           THEN
              LEFT
        ENDIF
        MENU {Field} {Justify} ENTER SELECT CF["Justify"]
  ENDIF
ENDPROC ;---ApplyJustification(AdjustCursorPosition)

PROC GetDateFormat()
  ;Select the date format for the date being pasted
  WHILE NOT SUBSTR(MenuChoice(),
      SEARCH(" ",MENUCHOICE())+1 ,
      LEN(MENUCHOICE())
      - SEARCH(" ",MENUCHOICE())+1 ) = CF["FieldFormat"]
     KEYPRESS MenuMove
  ENDWHILE
  ENTER
  ENTER
ENDPROC ;--- GetDateFormat()

PROC ErrorDialog(TheErrorMessage)
  PRIVATE Choice
  ;Dialog box for errors
  IF IsStandardMode
    THEN
      SHOWDIALOG "Error"
        @ 8, 10 HEIGHT 6 WIDTH 60
        @ 0,0 ?? FORMAT("W58,AC",TheErrorMessage)
        PUSHBUTTON
          @ 2, 24 WIDTH 10
         "OK" OK VALUE True TAG "OKButton" TO Retval
      ENDDIALOG
    ELSE
      MESSAGE TheErrorMessage
      SHOWMENU
        "Ok":""
      TO Choice
      MESSAGE ""
  ENDIF
ENDPROC ;--- ErrorDialog(TheErrorMessage)

PROC Display()
   ;Display field attributes array contents
   CANVAS OFF
   PAINTCANVAS ATTRIBUTE 7  3,10,23,76
   PAINTCANVAS FILL(" ") ATTRIBUTE 78 2,8,22,74
   CANVAS ON
   STYLE ATTRIBUTE 78
   @ 2,17 ?? "Array assignments from the last field copied"
   SETMARGIN 12
   @ 3,0
   ? "CF[\"Alpha\"]          = "
   ? "CF[\"AlwaysSign\"]     = "
   ? "CF[\"Commas\"]         = "
   ? "CF[\"Currency\"]       = "
   ? "CF[\"Decimals\"]       = "
   ? "CF[\"Digits\"]         = "
   ? "CF[\"FieldFormat\"]    = "
   ? "CF[\"Grouped\"]        = "
   ? "CF[\"International\"]  = "
   ? "CF[\"Justify\"]        = "
   ? "CF[\"NameOfField\"]    = "
   ? "CF[\"NegParens\"]      = "
   ? "CF[\"PlaceType\"]      = "
   ? "CF[\"PointNoPennies\"] = "
   ? "CF[\"StatType\"]       = "
   ? "CF[\"SummaryType\"]    = "
   ? "CF[\"Wrapped\"]        = "
   ? "CF[\"WrappedTimes\"]   = "
   STYLE ATTRIBUTE 79
   SETMARGIN 35
   @ 3,0
   IF ISASSIGNED(CF["Alpha"]) THEN ? FORMAT("W33,AL",STRVAL(CF["Alpha"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["AlwaysSign"]) THEN ? FORMAT("W33,AL",STRVAL(CF["AlwaysSign"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["Commas"]) THEN ? FORMAT("W33,AL",STRVAL(CF["Commas"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["Currency"]) THEN ? FORMAT("W33,AL",STRVAL(CF["Currency"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["Decimals"]) THEN ? FORMAT("W33,AL",STRVAL(CF["Decimals"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["Digits"]) THEN ? FORMAT("W33,AL",STRVAL(CF["Digits"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["FieldFormat"]) THEN ? FORMAT("W32,AL",STRVAL(CF["FieldFormat"]))
      IF LEN(CF["FieldFormat"]) > 32 THEN ?? "..." ENDIF
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["Grouped"]) THEN ? FORMAT("W33,AL",STRVAL(CF["Grouped"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["International"]) THEN ? FORMAT("W33,AL",STRVAL(CF["International"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["Justify"]) THEN ? FORMAT("W33,AL",STRVAL(CF["Justify"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["NameOfField"]) THEN ? FORMAT("W32,AL",STRVAL(CF["NameOfField"]))
      IF LEN(CF["NameOfField"]) > 32 THEN ?? "..." ENDIF
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["NegParens"]) THEN ? FORMAT("W33,AL",STRVAL(CF["NegParens"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["PlaceType"]) THEN ? FORMAT("W33,AL",STRVAL(CF["PlaceType"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["PointNoPennies"]) THEN ? FORMAT("W33,AL",STRVAL(CF["PointNoPennies"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["StatType"]) THEN ? FORMAT("W33,AL",STRVAL(CF["StatType"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["SummaryType"]) THEN ? FORMAT("W33,AL",STRVAL(CF["SummaryType"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["Wrapped"]) THEN ? FORMAT("W33,AL",STRVAL(CF["Wrapped"]))
      ELSE ? "Unassigned" ENDIF
   IF ISASSIGNED(CF["WrappedTimes"]) THEN ? FORMAT("W33,AL",STRVAL(CF["WrappedTimes"]))
      ELSE ? "Unassigned" ENDIF
   STYLE ATTRIBUTE 78
   SETMARGIN 0
   @22,28 ?? "Press any key to continue"
   STYLE
   a=GETCHAR()
   QUIT
ENDPROC ;--- Display()

PROC HelpScr()
  ;Display a help screen
  CLEAR
  @ 0,0
  STYLE REVERSE
  ?? FORMAT("W80,AC","Using CopyFld4 (version 1.1)")
  STYLE
  @ 1,0
  TEXT

      To copy a field in the report generator:
           Move your cursor onto the field you want to copy.
           Play CopyFld4 and select {Copy} from the CopyFld4 menu.

      To paste a field that you have copied:
           Move your cursor to the position in the report where you
          want place the copied field.
           Play CopyFld4 and select {Paste} from the CopyFld4 menu.

      Note: The following special conditions affect how CopyFld4 works:
           CopyFld4 assumes that any field named "Current date",
            "Current time", "Current page number", or
            "Current record number" is a report field, not a regular field.
           CopyFld4 assumes that any field whose name begins "Maximum for",
            "Minimum for","Average for", "Average of", "Total for", or 
            "Count of" is a summary field, not a regular field.

      CopyFld4 makes use of a dynamic array named CF.  Select Reset to release.



  ENDTEXT
  STYLE REVERSE
  ? FORMAT("W80,AC","Press any key to continue")
  STYLE
  WaitKey()
  QUIT  "Thank you for using CopyFld4."
ENDPROC ;--- HelpScr()

PROC Modify()
  ;Modify the copied field
  SWITCH
    CASE NOT ISASSIGNED(CF["FieldFormat"]):
      ErrorDialog("Copy a field before selecting Modify")
      QUIT
    CASE SUBSTR(CF["FieldFormat"],1,1) = "A":
      ModifyAlphaField()
    CASE SUBSTR(CF["FieldFormat"],1,1) ="(" OR
         SUBSTR(CF["FieldFormat"],1,1) = CHR(241) OR
         SUBSTR(CF["FieldFormat"],1,1) = "9":
      ModifyNumericField()
    CASE UPPER(SUBSTR(CF["FieldFormat"],1,1)) = "H":
      ModifyTimeField()
    CASE UPPER(SUBSTR(CF["FieldFormat"],1,1)) = "D" OR
         UPPER(SUBSTR(CF["FieldFormat"],1,1)) = "M" OR
         UPPER(SUBSTR(CF["FieldFormat"],1,1)) = "Y":
      ModifyDateField()
    OTHERWISE:
      NewCalculation()
  ENDSWITCH
ENDPROC ;--- Modify()    

PROC InfoScr()
  ;Display the credits screen
  CLEAR
  @ 0,0
  STYLE REVERSE
  ?? FORMAT("W80,AC","Credits and Information for CopyFld4 (version 1.1)")
  STYLE
  @ 1,0
  TEXT
         Written by Cary Jensen of Data Works of Houston, co-author of
             PAL for Paradox 4 Made Easy (Osborne/McGraw-Hill) and
                 The Paradox Instant Reference books (Sybex).

   Data Works of Houston provides Paradox and Paradox for Windows training,
   consultation, and application development.  Customized classes available.
                          ķ
                            Data Works of Houston  
                               (713) 359-3311      
                          Ľ
                                  *  *  *  *
      Invoke CopyFld4 using a "hot-key."  This is done using the SETKEY command.
      From your init script, or from the {MiniScript} selection from the
      PAL menu [ALT][F10], play the following command at the beginning of
      your Paradox session:

      SETKEY "F23" READLIB "C:\\PDOX40\\CopyTool" CopyFld4 CopyFld4()

      After you issue this command, CopyFld4 can be invoked by pressing CTRL-F3.
      To assign CopyFld4 to any other key, replace "F23" in the SETKEY command
      with any other valid PAL keycode or key name.  If Copytool.lib is stored
      in a directory other than C:\PDOX40, use that path in the SETKEY command.
  ENDTEXT
  STYLE REVERSE
  ? FORMAT("W80,AC","Press any key to continue")
  STYLE
  WaitKey()
  QUIT  "Thank you for using CopyFld4"
ENDPROC ;--- InfoScr()

PROC ResetArray()
   RELEASE VARS CF
   QUIT "The CopyFld4 Dynamic array has been reset."
ENDPROC ;--- ResetArray()

PROC CopyFld()
   ;Copy the current field
   IF FIELDINFO()=""
      THEN
         QUIT "Your cursor must be on a field."
      ELSE
         STYLE ATTRIBUTE 79
         @ 24,63 ?? " Copying field"
         STYLE ATTRIBUTE 207
         ?? "..."
         STYLE
         DYNARRAY CF[]
         CF["Alpha"] = False
         CF["Commas"] = False
         CF["Currency"] = False
         CF["International"] = False
         CF["PointNoPennies"] = False
         CF["Justify"] = "None"
         CF["NegParens"] = False
         CF["Wrapped"] = False
         CF["WrappedTimes"] = 0
         CF["NameOfField"]=FIELDINFO()
   ENDIF

   ;move to the first character of the field
   StartColumn=COLNO()
   LEFT
   ;If FIELDINFO is no longer the name, we moved off the field.
   ;If the column hasn't changed, we cannot move LEFT any more.
   WHILE FIELDINFO()=CF["NameOfField"] AND NOT StartColumn=COLNO()
      StartColumn=COLNO()
      LEFT SYNCCURSOR
   ENDWHILE
   ;If cursor moved with last LEFT, move
   ;back onto the field with a RIGHT
   IF NOT StartColumn=COLNO()
      THEN
         RIGHT
   ENDIF
   CF["NegParens"] = CURSORCHAR()="("
   CF["AlwaysSign"] = CURSORCHAR() = CHR(241)
   CF["FieldFormat"]=""
   ;Determine the field format.
   WHILE FIELDINFO()=CF["NameOfField"]
      CF["FieldFormat"]=CF["FieldFormat"]+CURSORCHAR()
      StartingColumn=COLNO()
      RIGHT
      IF StartingColumn=COLNO()
         THEN
            QUITLOOP
      ENDIF
   ENDWHILE
   ;Re-wind to the first of the field
   FOR i FROM 1 to LEN(CF["FieldFormat"])
      LEFT
   ENDFOR

   CF["Alpha"] = CURSORCHAR() = "A"
   ;If the field is alphanumeric
   ;see if the field is wordwrapped
   IF CF["Alpha"]
      THEN
         IF NOT SEARCH(", wrap:",CF["NameOfField"]) = 0
            THEN
               CF["Wrapped"] = True
               Position = SEARCH(", wrap:",CF["NameOfField"])
               CF["WrappedTimes"] = NumVal(SubStr(CF["NameOfField"],
                              Position+7, LEN(CF["NameOfField"])))
               IF CF["WrappedTimes"] = "Error"
                  THEN CF["WrappedTimes"] = "v"
               ENDIF
               CF["NameOfField"] = SubStr(CF["NameOfField"], 1, Position-1)
         ENDIF
   ENDIF

   ;Determine if the field is justified.
   NameLength=LEN(CF["NameOfField"])
   SWITCH
      CASE NameLength > 5 AND SUBSTR(CF["NameOfField"],
         NameLength-5, NameLength) = ", left":
         CF["NameOfField"]= SUBSTR(CF["NameOfField"], 1, NameLength-6)
         CF["Justify"] = "Left"
      CASE NameLength > 6 AND SUBSTR(CF["NameOfField"],
         NameLength-6, NameLength) = ", right":
         CF["NameOfField"]= SUBSTR(CF["NameOfField"], 1, NameLength-7)
         CF["Justify"] = "Right"
      CASE NameLength > 7 AND SUBSTR(CF["NameOfField"],
         NameLength-7, NameLength) = ", center":
         CF["NameOfField"]= SUBSTR(CF["NameOfField"], 1, NameLength-8)
         CF["Justify"] = "Center"
      OTHERWISE:
         CF["Justify"] = "None"
   ENDSWITCH


   ;Check for the various summary field types.  If the
   ;field is a summary field, assign the appropriate variables
   SWITCH
      CASE SUBSTR(CF["NameOfField"],1,12) = "Maximum for ":
         CF["StatType"]="High"
         CF["PlaceType"]="Summary"
         CF["Grouped"]=GroupedStatus()
         CF["SummaryType"]=SummaryType()
         CF["NameOfField"] = SubStr(CF["NameOfField"],13, LEN(CF["NameOfField"]))
         GetFieldSize()
      CASE SUBSTR(CF["NameOfField"],1,12) = "Minimum for ":
         CF["StatType"]="Low"
         CF["PlaceType"]="Summary"
         CF["Grouped"]=GroupedStatus()
         CF["SummaryType"]=SummaryType()
         CF["NameOfField"] = SubStr(CF["NameOfField"],13, LEN(CF["NameOfField"]))
         GetFieldSize()
      CASE SUBSTR(CF["NameOfField"],1,12) = "Average for ":
         CF["StatType"]="Average"
         CF["PlaceType"]="Summary"
         CF["Grouped"]=GroupedStatus()
         CF["SummaryType"]=SummaryType()
         CF["NameOfField"] = SubStr(CF["NameOfField"],13, LEN(CF["NameOfField"]))
         GetFieldSize()
      CASE SUBSTR(CF["NameOfField"],1,11) = "Average of ":
         CF["StatType"]="Average"
         CF["PlaceType"]="Summary"
         CF["Grouped"]=GroupedStatus()
         CF["SummaryType"]=SummaryType()
         CF["NameOfField"] = SubStr(CF["NameOfField"],12, LEN(CF["NameOfField"]))
         GetFieldSize()
      CASE SUBSTR(CF["NameOfField"],1,10) = "Total for ":
         CF["StatType"]="Sum"
         CF["PlaceType"]="Summary"
         CF["Grouped"]=GroupedStatus()
         CF["SummaryType"]=SummaryType()
         CF["NameOfField"] = SubStr(CF["NameOfField"],11, LEN(CF["NameOfField"]))
         GetFieldSize()
      CASE SUBSTR(CF["NameOfField"],1,9) = "Count of ":
         CF["StatType"]="Count"
         CF["PlaceType"]="Summary"
         CF["Grouped"]=GroupedStatus()
         CF["SummaryType"]=SummaryType()
         CF["NameOfField"] = SubStr(CF["NameOfField"],10, LEN(CF["NameOfField"]))
         GetFieldSize()
      ;The field is not a summary field.  It must then either
      ;be a regular field or a calculated field.
      OTHERWISE:
         IF SummaryType()="Calculated"
            THEN
               CF["PlaceType"]="Calculated"
               GetFieldSize()
            ELSE
               ;If the field is a regular field, make sure
               ;that it is not a report field.
               SWITCH
                  CASE CF["NameOfField"] = "Current date":
                     CF["PlaceType"]="Report field"
                     CF["StatType"]="Date"
                  CASE CF["NameOfField"] = "Current time":
                     CF["PlaceType"]="Report field"
                     CF["StatType"]="Time"
                  CASE CF["NameOfField"] = "Current page number":
                     GetFieldSize()
                     CF["PlaceType"]="Report field"
                     CF["StatType"]="Page"
                  CASE CF["NameOfField"] = "Current record number":
                     GetFieldSize()
                     CF["PlaceType"]="Report field"
                     CF["StatType"]="#Record"
                     CF["Grouped"]=False
                  CASE CF["NameOfField"] = "Current record number, per group":
                     GetFieldSize()
                     CF["PlaceType"]="Report field"
                     CF["Grouped"]=True
                     CF["StatType"]="#Record"
                  OTHERWISE: ;its a regular field
                     CF["PlaceType"]="Regular"
                     GetFieldSize()
               ENDSWITCH
         ENDIF
   ENDSWITCH

   QUIT "Field has been copied"
ENDPROC ;--- CopyFld()

PROC PasteFld()
   ;Paste the previously copied field
   IF NOT CURSORCHAR()=" "
      THEN
         QUIT "Cursor must be on a blank space in order to paste a field."
   ENDIF
   STYLE ATTRIBUTE 79
   @ 24,63 ?? " Pasting field"
   STYLE ATTRIBUTE 207
   ?? "..."
   STYLE
   Insert = IsInsertMode()
   ;Turn off insert mode if it is on.
   IF Insert
      THEN
         INS
   ENDIF

   WHILE True
      SWITCH
         CASE CF["PlaceType"]="Regular":
            MENU {Field} {Place} {Regular}
               ;It is possible that a field could be identified
               ;as a regular field although it is really a calculated
               ;field that uses functions or variables.  IsRegular()
               ;ensures that the field is a regular field.  If it is not,
               ;CF["PlaceType"] is assigned "Calculated" and False is returned
               IF NOT IsRegular()
                  THEN
                     LOOP
               ENDIF
               IF MENUCHOICE()="1) mm/dd/yy"
                  ;The field is a date field.  Select the correct format.
                  THEN
                     GetDateFormat()
                     ApplyJustification(True)
                     ;All done for regular field of type date.
                     QUITLOOP
               ENDIF
               ;If the field is not of the type Date, processing
               ;will continue after the ENDSWITCH.
         CASE CF["PlaceType"]="Report field":
            MENU {Field} {Place} SELECT CF["StatType"]
            SWITCH
               CASE CF["StatType"]="Date":
                  ;If the report field is Date,  select the correct format.
                  GetDateFormat()
                  ApplyJustification(True)
                  QUITLOOP
               CASE CF["StatType"]="Time":
                  ;If the report field is Time,  select the correct format.
                  WHILE NOT MATCH(MENUCHOICE(), ".."+ CF["FieldFormat"] +"..")
                     KEYPRESS MenuMove
                  ENDWHILE
                  ENTER
                  ENTER
                  ApplyJustification(True)
                  QUITLOOP
               CASE CF["StatType"]="#Record":
                     IF CF["Grouped"]
                        THEN
                           {Per-Group}
                        ELSE
                           {Overall}
                     ENDIF
               OTHERWISE:
            ENDSWITCH
            ;If the report field is type #Record, placement will continue
            ;after the ENDSWITCH
         CASE CF["PlaceType"]="Summary":
            MENU {Field} {Place} {Summary}
            SELECT CF["SummaryType"]
            IF CF["SummaryType"]="Calculated"
               THEN
                  TYPEIN CF["NameOfField"] ENTER
               ELSE
                  SELECT CF["NameOfField"]
            ENDIF
            SELECT CF["StatType"]
            IF CF["Grouped"]
               THEN
                  {PerGroup}
               ELSE
                  {Overall}
            ENDIF
            ;If the summary field is type date, select correct format.
            IF MENUCHOICE()="1) mm/dd/yy"
               THEN
                  GetDateFormat()
                  ApplyJustification(True)
                  QUITLOOP
            ENDIF
         CASE CF["PlaceType"]="Calculated":
            MENU {Field} {Place} {Calculated}
            TYPEIN CF["NameOfField"] ENTER
            ;If calculated field is date, select correct format.
            IF MENUCHOICE()="1) mm/dd/yy"
               THEN
                  GetDateFormat()
                  ApplyJustification(True)
                  QUITLOOP
            ENDIF
      ENDSWITCH

      StartingColumn = COLNO()
      ENTER
      CurrentColumn = COLNO()
      HOME
       IF NOT StartingColumn=COLNO() ;AND NOT CF["AlwaysSign"]
          THEN
             CF["Currency"] = True
          ELSE
             CF["Currency"] = False
       ENDIF

      ;=================================;
      ;We know that the field is a number,
      ;CF["Currency"], or CF["Alpha"]numeric field.
      ;=================================;

      For I from 2 to CF["Digits"]
         Right
      ENDFOR
      ENTER

      If Not CF["Decimals"]=0
         THEN
            HOME
            RIGHT
            FOR I FROM 1 TO CF["Decimals"]
                RIGHT
            ENDFOR
         ELSE
             IF CF["Currency"]
                THEN
                   HOME
             ENDIF
            IF CF["PointNoPennies"]
              THEN
                 RIGHT
            ENDIF
      ENDIF

      IF NOT (CF["Alpha"] OR CF["PlaceType"] = "Report field")
         THEN
            ;The field is a CF["Currency"] or numeric field.
            ;We need to issue one last ENTER.
            ENTER
            ;Now we need to set the appropriate formatting features.
            ;=================================;
            ;Setting digit/decimal delimiter notation.
            MENU {Field} {Reformat} ENTER {International}
            IF CF["International"]
               THEN
                  {InternationalConvention}
               ELSE
                  {U.S.Convention}
            ENDIF
            ;Setting commas.
            MENU {Field} {Reformat} ENTER {Commas}
            IF CF["Commas"]
               THEN
                  {Commas}
               ELSE
                  {NoCommas}
            ENDIF
            ;Setting sign convention.
            MENU {Field} {Reformat} ENTER {Sign-Convention}
            IF NOT CF["NegParens"]
               THEN
                  IF CF["AlwaysSign"]
                     THEN
                        {AlwaysSign}
                     ELSE
                        {NegativeOnly}
                  ENDIF
              ELSE
                {ParenNegative}
            ENDIF
      ENDIF

      ApplyJustification(False)

      IF CF["Wrapped"]
         THEN
            MENU {Field} {WordWrap} ENTER CTRLBACKSPACE
            TYPEIN CF["WrappedTimes"] ENTER
      ENDIF

      ;All done!
      QUITLOOP
   ENDWHILE
   ;If insert mode was on, turn it back on.
   IF Insert
      THEN
         INS
   ENDIF
   RELEASE VARS Choice, StartingColumn, i, ii, CurrentColumn,
                StartColumn, TempDigits, Insert
   QUIT "Field has been pasted"
ENDPROC ;--- PasteFld()

PROC CheckCalc(EventType, TagValue, EventValue, ElementValue)
  ;Verify an edited calculation
  PRIVATE Insert, Acceptable, CurrentPage, CurrentCol
  SWITCH
    CASE EventType = "ACCEPT":
      CurrentPage = PAGENO() CurrentCol = COLNO()
      IF ISINSERTMODE()
        THEN
          Insert = True
        ELSE 
          Insert = False
          INS
      ENDIF
      CTRLHOME ENTER UP
      MENU {Field} {Place} {Calculated} TYPEIN Calculation ENTER
      ;Check for a blank value
      IF NOT MENUCHOICE() = ""
        THEN
          ;The calculation is bad.
          ErrorDialog("Syntax error in expression")
          SELECTCONTROL "Accept"
          Acceptable =  False
        ELSE
          Acceptable = True
      ENDIF
      ;Cleanup after testing
      MENU ESC        ;Ensure the cursor is on the report  
      DELETELINE      ;Delete the line inserted for testing the calculation
      IF NOT Insert   ;Ensure insert mode is in original state
        THEN INS
      ENDIF
      ;Return the cursor to its position prior to calculation testing
      WHILE NOT CurrentPage = PAGENO()
        RIGHT      
      ENDWHILE
      WHILE NOT CurrentCol = COLNO()
        RIGHT
      ENDWHILE
      RETURN Acceptable
  ENDSWITCH 
ENDPROC ;--- CheckCalc(EventType, TagValue, EventValue, ElementValue)

PROC EditCalculation()
  ;Permit a calculation to be edited.
  PRIVATE Calculation, CurrentCol, CurrentPage, Insert
  Calculation = CF["NameOfField"]
  SHOWDIALOG "Modify the calculation"
    PROC "CheckCalc" TRIGGER "ACCEPT"
    @ 7,7 HEIGHT 7 WIDTH 69
    @ 1,2 ?? "Expression:"
    ACCEPT @1,14 WIDTH 51 "A175" 
      TAG "Accept" TO Calculation
    PUSHBUTTON @ 3,24 WIDTH 10
      "OK" OK DEFAULT VALUE True TAG "OKButton" TO Retval
    PUSHBUTTON @ 3,36 WIDTH 10
      "Cancel" CANCEL VALUE False TAG "CancelButton" TO Retval
  ENDDIALOG     

  IF Retval AND NOT (Calculation = CF["NameOfField"])
    THEN
      ;A new calculation has been specified
      CF["NameOfField"] = Calculation
      RETURN True
    ELSE
      RETURN False
  ENDIF
ENDPROC ;--- EditCalculation()

PROC ProcessAlphaModification(EventType, TagValue, EventValue, EventVar)
  ;Dialog proc for an alpha field modifcation
  SWITCH
    CASE EventType = "OPEN":
      SWITCH
        CASE CF["Wrapped"]:
          Wrap = 1
          Times = CF["WrappedTimes"]
        OTHERWISE:
          Wrap = 2
      ENDSWITCH
      SWITCH
        CASE CF["Justify"] = "Left":
          Justify = 1
        CASE CF["Justify"] = "Center":
          Justify = 2
        CASE CF["Justify"] = "Right":
          Justify = 3
        CASE CF["Justify"] = "None":
          Justify = 4
      ENDSWITCH
      REFRESHDIALOG
      RETURN

    CASE EventType = "ACCEPT":
      SWITCH
        CASE Wrap = 1:
          IF Times=""
            THEN
              ErrorDialog("Wrapped times not specified")
              RETURN False
            ELSE
              CF["WrappedTimes"] = Times
              CF["Wrapped"] = True
          ENDIF
        CASE Wrap = 2:
          CF["Wrapped"] = False
          CF["WrappedTimes"] = ""
      ENDSWITCH
      SWITCH
        CASE Justify = 1:
          CF["Justify"] = "Left"
        CASE Justify = 2:
          CF["Justify"] = "Center"          
        CASE Justify = 3:
          CF["Justify"] = "Right"          
        CASE Justify = 4:
          CF["Justify"] = "None"
      ENDSWITCH
  ENDSWITCH 
  RETURN True
ENDPROC ;--- ProcessAlphaModification(EventType, TagValue, EventValue, EventVar)

PROC ModifyAlphaField()
;Modify an alpha field
PRIVATE Wrap, Times, Justify, OKButton, CancelButton, EditCalcButton

  SHOWDIALOG "Modify Alphanumeric Field"
     PROC "ProcessAlphaModification" TRIGGER "OPEN","ACCEPT"
     @2,21 HEIGHT 20 WIDTH 38

     PUSHBUTTON @16,4 WIDTH 10
        "Ok"
        OK 
        DEFAULT
        VALUE "True"
        TAG "OKButton"
        TO OKButton

     PUSHBUTTON @16,20 WIDTH 10
        "Cancel"
        CANCEL 
        VALUE "False"
        TAG "CancelButton"
        TO CancelButton

     PUSHBUTTON @13,7 WIDTH 20
        "~E~dit Calculation"
        VALUE EditCalculation()
        TAG "EditCalcButton"
        TO EditCalcButton
     
     LABEL @1,7 "~J~ustify" FOR "Justify"
     RADIOBUTTONS @2,7 HEIGHT 4 WIDTH 13
        "Left",
        "Center",
        "Right",
        "Default"
        TAG "Justify"
        TO Justify

     LABEL @7,7 "~W~ord wrapping" FOR "Wrap"
     RADIOBUTTONS @8,7 HEIGHT 2 WIDTH 16
        "WordWrap",
        "NoWordWrap"
        TAG "Wrap"
        TO Wrap

     LABEL @11,7 "Wrapped ~T~imes" FOR "Times"
     ACCEPT @ 11,22 WIDTH 5 "a2" PICTURE "{#[#],V}" TAG "Times" TO Times

  ENDDIALOG
ENDPROC ;--- ModifyAlphaField()

PROC ProcessDateModification(EventType, TagValue, EventValue, EventVar)
  ;Dialog proc of a date field modification
  SWITCH
    CASE EventType = "OPEN":
      SWITCH
        CASE CF["FieldFormat"] ="mm/dd/yy":
          DateFormat = 1       
        CASE CF["FieldFormat"] = "M o n t h dd, yyyy":
          DateFormat = 2
        CASE CF["FieldFormat"] = "mm/dd":
          DateFormat = 3
        CASE CF["FieldFormat"] = "mm/yy":
          DateFormat = 4
        CASE CF["FieldFormat"] = "dd-Mon-yy":
          DateFormat = 5
        CASE CF["FieldFormat"] = "Mon yy":
          DateFormat = 6
        CASE CF["FieldFormat"] = "dd-Mon-yyyy":
          DateFormat = 7
        CASE CF["FieldFormat"] = "mm/dd/yyyy":
          DateFormat = 8
        CASE CF["FieldFormat"] = "dd.mm.yy":
          DateFormat = 9
        CASE CF["FieldFormat"] = "dd/mm/yy":
          DateFormat = 10
        CASE CF["FieldFormat"] = "yy-mm-dd":
          DateFormat = 11
        CASE CF["FieldFormat"] = "yy.mm.dd":
          DateFormat = 12
      ENDSWITCH
      SWITCH
        CASE CF["Justify"] = "Left":
          Justify = 1
        CASE CF["Justify"] = "Center":
          Justify = 2
        CASE CF["Justify"] = "Right":
          Justify = 3
        CASE CF["Justify"] = "None":
          Justify = 4
      ENDSWITCH
      REFRESHDIALOG
      RETURN
       
    CASE EventType = "ACCEPT":
      SWITCH
        CASE DateFormat = 1:
          CF["FieldFormat"] ="mm/dd/yy"
        CASE DateFormat = 2:
          CF["FieldFormat"] = "M o n t h dd, yyyy"
        CASE DateFormat = 3:
          CF["FieldFormat"] = "mm/dd"
        CASE DateFormat = 4:
          CF["FieldFormat"] = "mm/yy"
        CASE DateFormat = 5:
          CF["FieldFormat"] = "dd-Mon-yy"
        CASE DateFormat = 6:
          CF["FieldFormat"] = "Mon yy"
        CASE DateFormat = 7:
          CF["FieldFormat"] = "dd-Mon-yyyy"
        CASE DateFormat = 8:
          CF["FieldFormat"] = "mm/dd/yyyy"
        CASE DateFormat = 9:
          CF["FieldFormat"] = "dd.mm.yy"
        CASE DateFormat = 10:
          CF["FieldFormat"] = "dd/mm/yy"
        CASE DateFormat = 11:
          CF["FieldFormat"] = "yy-mm-dd"
        CASE DateFormat = 12:
          CF["FieldFormat"] = "yy.mm.dd"
      ENDSWITCH
      SWITCH
        CASE Justify = 1:
          CF["Justify"] = "Left"
        CASE Justify = 2:
          CF["Justify"] = "Center"
        CASE Justify = 3:
          CF["Justify"] = "Right"
        CASE Justify = 4:
          CF["Justify"] = "None"
      ENDSWITCH
        
  ENDSWITCH 
  RETURN True
ENDPROC ;--- ProcessDateModification(EventType, TagValue, EventValue, EventVar)

PROC ModifyDateField()
;Modify a date field
PRIVATE CancelButton, OKButton, DateFormat, Justify
  SHOWDIALOG "Modify Date Field"
     PROC "ProcessDateModification" TRIGGER "OPEN","ACCEPT"
     @2,15 HEIGHT 21 WIDTH 51

     PUSHBUTTON @15,16 WIDTH 20
        "~E~dit Calculation"
        VALUE EditCalculation()
        TAG "EditCalcButton"
        TO EditCalcButton

     PUSHBUTTON @17,13 WIDTH 10
        "Ok"
        OK 
        DEFAULT
        VALUE "True"
        TAG "OKButton"
        TO OKButton

     PUSHBUTTON @17,29 WIDTH 10
        "Cancel"
        CANCEL 
        VALUE "False"
        TAG "CancelButton"
        TO CancelButton

     LABEL @1,31 "~J~ustification" FOR "Justify"
     RADIOBUTTONS @2,31 HEIGHT 4 WIDTH 13
        "Left",
        "Center",
        "Right",
        "Default"
        TAG "Justify"
        TO Justify

     LABEL @1,3 "~F~ormat" FOR "DateFormat"
     RADIOBUTTONS @2,3 HEIGHT 12 WIDTH 27
        "1) mm/dd/yy",
        "2) M o n t h dd, yyyy",
        "3) mm/dd",
        "4) mm/yy",
        "5) dd-Mon-yy",
        "6) Mon yy",
        "7) dd-Mon-yyyy",
        "8) mm/dd/yyyy",
        "9) dd.mm.yy",
        "10) dd/mm/yy",
        "11) yy-mm-dd",
        "12) yy.mm.dd"
        TAG "DateFormat"
        TO DateFormat
  ENDDIALOG
ENDPROC ;--- ModifyDateField()

PROC ProcessTimeModification(EventType, TagValue, EventValue, EventVar)
;Dialog proc for a time field modification
SWITCH
  CASE EventType = "OPEN":
    SWITCH
      CASE CF["FieldFormat"] = "hh:mm pm":
        TimeFormat = 1
      CASE CF["FieldFormat"] = "hh:mm:ss":
        TimeFormat = 2
    ENDSWITCH
    REFRESHDIALOG
    RETURN
       
  CASE EventType = "ACCEPT":
    SWITCH
      CASE TimeFormat = 1:
        CF["FieldFormat"] = "hh:mm pm"
      CASE TimeFormat = 2:
      CF["FieldFormat"] = "hh:mm:ss"
    ENDSWITCH
ENDSWITCH  
RETURN True
ENDPROC ;--- ProcessTimeModification(EventType, TagValue, EventValue, EventVar)

PROC ModifyTimeField()
;Modify a time field
PRIVATE TimeFormat, CancelButton, OKButton
  SHOWDIALOG "Modify Time Field"
     PROC  "ProcessTimeModification" TRIGGER "OPEN", "ACCEPT"  
     @5,23 HEIGHT 9 WIDTH 34

     PUSHBUTTON @5,5 WIDTH 10
        "Ok"
        OK 
        DEFAULT
        VALUE "True"
        TAG "OKButton"
        TO OKButton

     PUSHBUTTON @5,17 WIDTH 10
        "Cancel"
        CANCEL 
        VALUE "False"
        TAG "CancelButton"
        TO CancelButton

     LABEL @1,5 "~F~ormat" FOR "TimeFormat"
     RADIOBUTTONS @2,5 HEIGHT 2 WIDTH 26
        "hh:mm pm",
        "hh:mm:ss (military)"
        TAG "TimeFormat"
        TO TimeFormat
  ENDDIALOG
ENDPROC ;--- ModifyTimeField()

PROC ProcessNumberModification(EventType, TagValue, EventValue, EventVar)
  ;Dialog proc for a numeric field modification
  SWITCH
    CASE EventType = "OPEN":
      SWITCH
        CASE CF["AlwaysSign"]:
          SignConvention = 3            
        CASE CF["NegParens"]:
          SignConvention = 2 
        OTHERWISE:
          SignConvention = 1           
      ENDSWITCH
      SWITCH
        CASE CF["Commas"]:
          Commas = 1
        OTHERWISE:
          Commas = 2
      ENDSWITCH
      SWITCH
        CASE CF["International"]:
          Convention = 2
        OTHERWISE:
          Convention = 1
      ENDSWITCH
      SWITCH
        CASE CF["Justify"] = "Left":
          Justify = 1
        CASE CF["Justify"] = "Center":
          Justify = 2
        CASE CF["Justify"] = "Right":
          Justify = 3
        CASE CF["Justify"] = "None":
          Justify = 4
      ENDSWITCH
      PointNoPennies = CF["PointNoPennies"]       
      NDigits = CF["Digits"]
      NDecimals = CF["Decimals"]     
      REFRESHDIALOG
      RETURN

    CASE EventType = "ACCEPT":
      IF PointNoPennies AND NDecimals <> 0
        THEN
          ErrorDialog("No decimals expected when PointNoPennies checked.")
          RETURN False
      ENDIF
      SWITCH
        CASE SignConvention = 3:
          CF["AlwaysSign"] = True
          CF["NegParens"] = False
        CASE SignConvention = 2:
          CF["NegParens"] = True
          CF["AlwaysSign"] = False
        OTHERWISE:
          CF["NegParens"] = False
          CF["AlwaysSign"] = False
      ENDSWITCH
      SWITCH
        CASE Commas = 1:
          CF["Commas"] = True
        OTHERWISE:
          CF["Commas"] = False
      ENDSWITCH
      SWITCH
        CASE Convention = 2:
          CF["International"] = True
        OTHERWISE:
          CF["International"] = False
      ENDSWITCH
      SWITCH
        CASE Justify = 1:
          CF["Justify"] = "Left"
        CASE Justify = 2:
          CF["Justify"] = "Center"
        CASE Justify = 3:
          CF["Justify"] = "Right"
        CASE Justify = 4:
          CF["Justify"] = "None"
      ENDSWITCH      
      CF["Digits"] = NDigits
      CF["Decimals"] = NDecimals
      CF["PointNoPennies"] = PointNoPennies
  ENDSWITCH 
  RETURN True
ENDPROC ;- ProcessNumberModification(EventType, TagValue, EventValue, EventVar)

PROC ModifyNumericField()
;Modify a numeric field
PRIVATE Commas, Convention, NDigits, NDecimals, OKButton, CancelButton,
        EditCalcButton, SignConvention, Justify, PointNoPennies
  SHOWDIALOG "Modify Numeric Field"
     PROC  "ProcessNumberModification" TRIGGER "OPEN","ACCEPT"
     @2,18 HEIGHT 21 WIDTH 44

     LABEL @ 1,2 "~S~ign" FOR "SignConvention"
     RADIOBUTTONS @2,2 HEIGHT 3 WIDTH 19
        "NegativeOnly",
        "ParenNegative",
        "AlwaysSign"
        TAG "SignConvention"
        TO SignConvention

     LABEL @ 6,2 "~C~ommas" FOR "Commas"
     RADIOBUTTONS @7,2 HEIGHT 2 WIDTH 14
        "Commas",
        "NoCommas"
        TAG "Commas"
        TO Commas

     LABEL @ 11,19 "~D~ecimals" FOR "NDecimals"
     ACCEPT @ 11,27 WIDTH 5 "S" MIN 0 MAX 15 TAG "NDecimals" TO NDecimals

     LABEL @ 11,2 "Digi~t~s" FOR "NDigits"
     ACCEPT @ 11,12 WIDTH 5 "S" MIN 0 MAX 15 TAG "NDigits" TO NDigits
     
     CHECKBOXES @ 13,19 HEIGHT 1 WIDTH 19 
       TAG "Pointless" "~P~ointNoPennies" TO PointNoPennies
     
     LABEL @ 1,23 "Con~v~ention" FOR "Convention"
     RADIOBUTTONS @2,23 HEIGHT 2 WIDTH 19
        "USConvention",
        "International"
        TAG "Convention"
        TO Convention

     PUSHBUTTON @17,8 WIDTH 10
        "Ok"
        OK 
        DEFAULT
        VALUE "True"
        TAG "OKButton"
        TO OKButton

     PUSHBUTTON @17,24 WIDTH 10
        "Cancel"
        CANCEL 
        VALUE "False"
        TAG "CancelButton"
        TO CancelButton

     PUSHBUTTON @15,10 WIDTH 20
        "~E~dit Calculation"
        VALUE EditCalculation()
        TAG "EditCalcButton"
        TO EditCalcButton

     LABEL @5,22 "~J~usification" FOR "Justify"
     RADIOBUTTONS @6,22 HEIGHT 4 WIDTH 13
        "Left",
        "Center",
        "Right",
        "Default"
        TAG "Justify"
        TO Justify

  ENDDIALOG
ENDPROC ;--- ModifyNumericField()

;Ŀ
;                                                                         
;                       M A I N   P R O G R A M                           
;                                                                         
;
;Determine compatiblilty mode
MENU HOME
IF MENUCHOICE() = CHR(240)
  THEN
    IsStandardMode = True
    MenuMove = "DOWN"
  ELSE
    IsStandardMode = False
    MenuMove = "RIGHT"
ENDIF
ESC

@ 24,0 CLEAR EOL
SWITCH
  CASE IsStandardMode:
  SWITCH
    CASE (NOT SYSMODE()= "Report") OR
       (NOT IsAssigned(CF["NameOfField"]) AND FIELDINFO()=""):
       SHOWPULLDOWN
          "Help"   :"Display help on how to use CopyFld4.":"Help",
          "Info"   :"Display credits and CopyFld4 information.":"Info",
          "Reset"  :"Release the dynamic array used by CopyFld4.":"Reset",
          "Display":"Display the contents of the dynamic array used by "+
                    "CopyFld4.":"Display",
          "Exit"   :"Exit CopyFld4.":"Exit"
       ENDMENU

    CASE IsAssigned(CF["NameOfField"]) AND FIELDINFO()="":
      SHOWPULLDOWN
        "Paste"  :"Paste the previously copied field "+
                  "to the location at the cursor.":"Paste",
        "Help"   :"Display help on how to use CopyFld4.":"Help",
        "Info"   :"Display credits and CopyFld4 information.":"Info",
        "Reset"  :"Release the dynamic array used by CopyFld4.":"Reset",
        "Display":"Display the contents of dynamic array used by "+
                  "CopyFld4.":"Display",
        "Modify" :"Modify one of more elements of the copied field.":"Modify",
        "Exit"   :"Return to the report designer.":"Exit"
      ENDMENU         
    OTHERWISE:
      SHOWPULLDOWN
        "Copy"   :"Copy the field at the cursor.":"Copy",
        "Help"   :"Display help on how to use CopyFld4.":"Help",
        "Info"   :"Display credits and CopyFld4 information.":"Info",
        "Reset"  :"Release the dynamic array used by CopyFld4.":"Reset",
        "Display":"Display the contents of the dynamic array used by "+
                  "CopyFld4.":"Display",
        "Modify" :"Modify one of more elements of the copied field.":"Modify",
        "Exit"   :"Return to the report designer.":"Exit"
      ENDMENU
  ENDSWITCH
  GETMENUSELECTION TO Choice
  CLEARPULLDOWN

  OTHERWISE:
  SWITCH
    CASE (NOT SYSMODE()= "Report") OR
       (NOT IsAssigned(CF["NameOfField"]) AND FIELDINFO()=""):
       SHOWMENU
          "Help"   :"Display help on how to use CopyFld4.",
          "Info"   :"Display credits and CopyFld4 information.",
          "Reset"  :"Release the dynamic array used by CopyFld4.",
          "Display":"Display the contents of the dynamic array used by "+
                    "CopyFld4.",
          "Exit"   :"Exit CopyFld4."
       TO Choice

    CASE IsAssigned(CF["NameOfField"]) AND FIELDINFO()="":
      SHOWMENU
        "Paste"  :"Paste the previously copied field "+
                  "to the location at the cursor.",
        "Help"   :"Display help on how to use CopyFld4.",
        "Info"   :"Display credits and CopyFld4 information.",
        "Reset"  :"Release the dynamic array used by CopyFld4.",
        "Display":"Display the contents of dynamic array used by "+
                  "CopyFld4.",
        "Exit"   :"Return to the report designer."
      TO Choice
    OTHERWISE:
      SHOWMENU
        "Copy"   :"Copy the field at the cursor.",
        "Help"   :"Display help on how to use CopyFld4.",
        "Info"   :"Display credits and CopyFld4 information.",
        "Reset"  :"Release the dynamic array used by CopyFld4.",
        "Display":"Display the contents of the dynamic array used by "+
                  "CopyFld4.",
        "Exit"   :"Return to the report designer."
      TO Choice
  ENDSWITCH
ENDSWITCH

SWITCH
   CASE Choice="Exit" OR Choice="Esc":
      CLEAR
      QUIT  "Thank you for using CopyFld4"

   CASE Choice = "Display": Display()

   CASE Choice = "Modify":  Modify()

   CASE Choice = "Help":    HelpScr()

   CASE Choice = "Info":    InfoScr()

   CASE Choice="Reset":     ResetArray()

   CASE Choice="Copy":      CopyFld()

   CASE Choice="Paste":     PasteFld()
  
ENDSWITCH
ENDPROC ;------------------------- CopyFld4 ------------------------------
IF NOT ISFILE(DIRECTORY()+"CopyTool.lib")
   THEN
      CREATELIB DIRECTORY()+"CopyTool"
   ELSE
      IF NUMVAL(FILEVERSION(DIRECTORY()+"CopyTool.lib")) < 4
        THEN
          QUIT "Problem. COPYTOOL.LIB is not a Paradox 4.0 library. "+
               "Cannot add CopyFld4. Re-run Copyfld4 in another directory "+
               "or delete COPYTOOL.LIB."
      ENDIF
ENDIF
Writelib DIRECTORY()+"copytool" CopyFld4
QUIT "COPYTOOL.LIB is stored in "+DIRECTORY()+"  Read CopyFld4.sc or README."
+"TXT for directions on how to play CopyFld4.  Press any key to continue."
