; Ŀ
;  Procedure:  EditTab(TableName, FormNumber, Height, Width)               
;  Purpose  :  Display the named table using the named form.  The table    
;  is placed in CoEdit mode, but the IMAGERIGHTS are set to readonly.      
;  If the user wishes to edit a record, they must press F9 to do so.       
;  Although this technique is restrictive, it permits the programmer to    
;  have the highest level of control over record locking and validation.   
;                                                                          
;    This script is part of a sample application described in the          
;    October 1993 issue of Paradox Information.  This script was written   
;    by Cary Jensen of Data Works of Houston.                              
;                                                                          
;    This script is for demonstration purposes only.  No representation    
;    is made concerning the appropriateness of this script for any other   
;    purpose.  If you use this script you agree to hold its author and     
;    Data Works of Houston free from any claim arising out of the use or   
;    misuse of this script.                                                
; ͵
;                              Cary Jensen                                 
;                          Data Works of Houston                           
;                             CIS 76307,1533                               
;                             (713) 359-3311                               
; ͵
; 
PROC EditTab(TableName, FormNumber, Height, Width)
PRIVATE WaitMessage

;Lock the table.
IF NOT TabLock("\""+Tablename+"\" PFL")
  THEN RETURN
ENDIF

IF NOT FormNumber = "F"
  THEN
    ;Verify that the form exists.  If not, use the Standard form.
    IF NOT ISFILE(TableName+".F"+FormNumber)
      THEN
        FormNuber = "F"
    ENDIF
ENDIF

;Enter the CoEdit mode, but do not permit changes.
COEDIT TableName
IMAGERIGHTS READONLY
;Put the tableview out of the way
WINDOW SETATTRIBUTES GETWINDOW() FROM TableWindow

PICKFORM FormNumber
;Define the characteristics of the form view
WinHandle = GETWINDOW()
WindowStyle["TITLE"] = Table()
WindowStyle["HEIGHT"] = Height
WindowStyle["WIDTH"] = Width
WINDOW SETATTRIBUTES WinHandle FROM WindowStyle

WaitMessage = ""

  WHILE True
    ;The following is a simplified version of the WAIT statement.  For
    ;a more powerful version of WAIT, one that makes use of a WAIT PROC,
    ;see your PAL documentation.
    WAIT TABLE
      PROMPT  "[Ctrl-Z]-Zoom     [Ins]-Insert     [Del]-Delete     [F9]-Edit     [Esc]-Return"
      MESSAGE WaitMessage
      UNTIL "F2", "DOS", "DOSBIG", "DEL","ESC","ZOOM","INS","DEL", "F9"

    WaitMesssage = ""

    SWITCH
      CASE Retval = "INS":
        ;If not in field view, insert a new record.
        IF ISFIELDVIEW()
          THEN INS
          ELSE
            IMAGERIGHTS
            INS
            ; Test for blank keys.
            LOCKRECORD
            IF NOT Retval
              THEN
                ;Another blank record already exists.  Cannot continue.
                DEL
                WaitMessage = "A blank record already exists.  Cannot "+
                              "add a new blank record."
                LOOP
              ELSE
                ;Change the new record to set its state to unposted
                [] = []
            ENDIF
            IMAGERIGHTS READONLY
            ;Edit the newly inserted record.
            EditRec()
        ENDIF

      CASE Retval = "DEL":
        ;If not in field view, get confirmation.
        IF ISFIELDVIEW()
          THEN DEL
          ELSE
            SHOWMENU
              "Cancel" :"Do not delete this record.",
              "Ok"     :"Delete this record."
              TO Choice
            IF Choice = "Ok"
              THEN
                IMAGERIGHTS
                DEL
                IMAGERIGHTS READONLY
            ENDIF
        ENDIF

      CASE Retval = "F9":
        ;Edit the current record.
        EditRec()

      CASE RetVal = "DOS" OR RetVal = "DOSBIG":
        BEEP
        LOOP

      CASE Retval = "ZOOM":
        ;Use ZoomFld to Zoom.  This is necessary since Esc is in
        ;the UNTIL key list.  If the native zoom where used, a user
        ;pressing Esc to leave Zoom would trigger the UNTIL.
        ZoomFld()
        IF NOT Retval = ""
          THEN
            WaitMessage = Retval
          ELSE
            WaitMessage = ""
        ENDIF

      CASE RetVal = "ESC":
        ;Leave.
        DO_IT!
        UNLOCK TableName PFL
        CLEARIMAGE
        RETURN

    ENDSWITCH
  ENDWHILE
ENDPROC ; --- EditTab(TableName, FormNumber, Height, Width) 
WRITELIB "Paradox" EditTab
RELEASE PROCS EditTab
